@@ -20,6 +20,9 @@ static struct debugfs_module {
       .func = do_debugfs_extfrag},
     { .name = "/sys/kernel/debug/zswap", .enabled = CONFIG_BOOLEAN_YES,
       .func = do_debugfs_zswap},
+    // Linux powercap metrics is here because it needs privilege to read each RAPL zone
+    { .name = "/sys/devices/virtual/powercap", .enabled = CONFIG_BOOLEAN_YES,
+      .func = do_sys_devices_virtual_powercap},
 
     // The terminator
     { .name = NULL, .enabled = CONFIG_BOOLEAN_NO, .func = NULL}
@@ -156,14 +159,8 @@ static void debugfs_parse_args(int argc, char **argv)
 
 int main(int argc, char **argv)
 {
-    // debug_flags = D_PROCFILE;
-    stderror = stderr;
-
-    // set the name for logging
-    program_name = "debugfs.plugin";
-
-    // disable syslog for debugfs.plugin
-    error_log_syslog = 0;
+    clocks_init();
+    nd_log_initialize_for_external_plugins("debugfs.plugin");
 
     netdata_configured_host_prefix = getenv("NETDATA_HOST_PREFIX");
     if (verify_netdata_host_prefix() == -1)
@@ -238,6 +235,13 @@ int main(int argc, char **argv)
             netdata_log_info("all modules are disabled, exiting...");
             return 1;
         }
+
+        fprintf(stdout, "\n");
+        fflush(stdout);
+        if (ferror(stdout) && errno == EPIPE) {
+            netdata_log_error("error writing to stdout: EPIPE. Exiting...");
+            return 1;
+        }
     }
 
     fprintf(stdout, "EXIT\n");
