@@ -52,15 +52,15 @@ static size_t network_interfaces_added = 0, network_interfaces_found = 0;
 
 static void network_interface_free(struct cgroup_network_interface *ifm) {
     if (likely(ifm->st_bandwidth))
-        rrdset_is_obsolete(ifm->st_bandwidth);
+        rrdset_is_obsolete___safe_from_collector_thread(ifm->st_bandwidth);
     if (likely(ifm->st_packets))
-        rrdset_is_obsolete(ifm->st_packets);
+        rrdset_is_obsolete___safe_from_collector_thread(ifm->st_packets);
     if (likely(ifm->st_errors))
-        rrdset_is_obsolete(ifm->st_errors);
+        rrdset_is_obsolete___safe_from_collector_thread(ifm->st_errors);
     if (likely(ifm->st_drops))
-        rrdset_is_obsolete(ifm->st_drops);
+        rrdset_is_obsolete___safe_from_collector_thread(ifm->st_drops);
     if (likely(ifm->st_events))
-        rrdset_is_obsolete(ifm->st_events);
+        rrdset_is_obsolete___safe_from_collector_thread(ifm->st_events);
 
     network_interfaces_added--;
     freez(ifm->name);
@@ -177,15 +177,15 @@ int do_getifaddrs(int update_every, usec_t dt) {
                                                         CONFIG_BOOLEAN_AUTO);
 
         excluded_interfaces = simple_pattern_create(
-                config_get(CONFIG_SECTION_GETIFADDRS, "disable by default interfaces matching", DEFAULT_EXCLUDED_INTERFACES)
-                , NULL
-                , SIMPLE_PATTERN_EXACT
-        );
+            config_get(CONFIG_SECTION_GETIFADDRS, "disable by default interfaces matching", DEFAULT_EXCLUDED_INTERFACES),
+            NULL,
+            SIMPLE_PATTERN_EXACT,
+            true);
         physical_interfaces = simple_pattern_create(
-                config_get(CONFIG_SECTION_GETIFADDRS, "set physical interfaces for system.net", DEFAULT_PHYSICAL_INTERFACES)
-                , NULL
-                , SIMPLE_PATTERN_EXACT
-        );
+            config_get(CONFIG_SECTION_GETIFADDRS, "set physical interfaces for system.net", DEFAULT_PHYSICAL_INTERFACES),
+            NULL,
+            SIMPLE_PATTERN_EXACT,
+            true);
     }
 
     if (likely(do_bandwidth_ipv4 || do_bandwidth_ipv6 || do_bandwidth || do_packets || do_errors || do_bandwidth_net || do_packets_net ||
