@@ -4,11 +4,11 @@
 #include <sched.h>
 
 char pidfile[FILENAME_MAX + 1] = "";
-char claimingdirectory[FILENAME_MAX + 1];
-char exepath[FILENAME_MAX + 1];
+char claiming_directory[FILENAME_MAX + 1];
+char netdata_exe_path[FILENAME_MAX + 1];
+char netdata_exe_file[FILENAME_MAX + 1];
 
-void get_netdata_execution_path(void)
-{
+void get_netdata_execution_path(void) {
     int ret;
     size_t exepath_size = 0;
     struct passwd *passwd = NULL;
@@ -17,30 +17,18 @@ void get_netdata_execution_path(void)
     passwd = getpwuid(getuid());
     user = (passwd && passwd->pw_name) ? passwd->pw_name : "";
 
-    exepath_size = sizeof(exepath) - 1;
-    ret = uv_exepath(exepath, &exepath_size);
+    exepath_size = sizeof(netdata_exe_file) - 1;
+    ret = uv_exepath(netdata_exe_file, &exepath_size);
     if (0 != ret) {
-        netdata_log_error("uv_exepath(\"%s\", %u) (user: %s) failed (%s).", exepath, (unsigned)exepath_size, user,
-              uv_strerror(ret));
+        netdata_log_error("uv_exepath(\"%s\", %u) (user: %s) failed (%s).", netdata_exe_file, (unsigned)exepath_size, user,
+                          uv_strerror(ret));
         fatal("Cannot start netdata without getting execution path.");
     }
-    exepath[exepath_size] = '\0';
-}
 
-static void chown_open_file(int fd, uid_t uid, gid_t gid) {
-    if(fd == -1) return;
+    netdata_exe_file[exepath_size] = '\0';
 
-    struct stat buf;
-
-    if(fstat(fd, &buf) == -1) {
-        netdata_log_error("Cannot fstat() fd %d", fd);
-        return;
-    }
-
-    if((buf.st_uid != uid || buf.st_gid != gid) && S_ISREG(buf.st_mode)) {
-        if(fchown(fd, uid, gid) == -1)
-            netdata_log_error("Cannot fchown() fd %d.", fd);
-    }
+    strcpy(netdata_exe_path, netdata_exe_file);
+    dirname(netdata_exe_path);
 }
 
 static void fix_directory_file_permissions(const char *dirname, uid_t uid, gid_t gid, bool recursive)
@@ -99,7 +87,7 @@ void prepare_required_directories(uid_t uid, gid_t gid) {
     change_dir_ownership(netdata_configured_varlib_dir, uid, gid, false);
     change_dir_ownership(netdata_configured_lock_dir, uid, gid, false);
     change_dir_ownership(netdata_configured_log_dir, uid, gid, false);
-    change_dir_ownership(claimingdirectory, uid, gid, false);
+    change_dir_ownership(claiming_directory, uid, gid, false);
 
     char filename[FILENAME_MAX + 1];
     snprintfz(filename, FILENAME_MAX, "%s/registry", netdata_configured_varlib_dir);
@@ -120,9 +108,6 @@ int become_user(const char *username, int pid_fd) {
     uid_t uid = pw->pw_uid;
     gid_t gid = pw->pw_gid;
 
-    if (am_i_root)
-        netdata_log_info("I am root, so checking permissions");
-
     prepare_required_directories(uid, gid);
 
     if(pidfile[0]) {
@@ -146,9 +131,9 @@ int become_user(const char *username, int pid_fd) {
         }
     }
 
+    nd_log_chown_log_files(uid, gid);
     chown_open_file(STDOUT_FILENO, uid, gid);
     chown_open_file(STDERR_FILENO, uid, gid);
-    chown_open_file(stdaccess_fd, uid, gid);
     chown_open_file(pid_fd, uid, gid);
 
     if(supplementary_groups && ngroups > 0) {
@@ -225,7 +210,7 @@ static void oom_score_adj(void) {
     // check the environment
     char *s = getenv("OOMScoreAdjust");
     if(!s || !*s) {
-        snprintfz(buf, 30, "%d", (int)wanted_score);
+        snprintfz(buf, sizeof(buf) - 1, "%d", (int)wanted_score);
         s = buf;
     }
 
@@ -260,7 +245,7 @@ static void oom_score_adj(void) {
     int written = 0;
     int fd = open("/proc/self/oom_score_adj", O_WRONLY);
     if(fd != -1) {
-        snprintfz(buf, 30, "%d", (int)wanted_score);
+        snprintfz(buf, sizeof(buf) - 1, "%d", (int)wanted_score);
         ssize_t len = strlen(buf);
         if(len > 0 && write(fd, buf, (size_t)len) == len) written = 1;
         close(fd);
@@ -516,7 +501,7 @@ int become_daemon(int dont_fork, const char *user)
     sched_setscheduler_set();
 
     // Set claiming directory based on user config directory with correct ownership
-    snprintfz(claimingdirectory, FILENAME_MAX, "%s/cloud.d", netdata_configured_varlib_dir);
+    snprintfz(claiming_directory, FILENAME_MAX, "%s/cloud.d", netdata_configured_varlib_dir);
 
     if(user && *user) {
         if(become_user(user, pidfd) != 0) {
