@@ -61,7 +61,7 @@ const struct netdata_static_thread static_threads_common[] = {
         .config_name = "netdata monitoring",
         .env_name = "NETDATA_INTERNALS_MONITORING",
         .global_variable = &global_statistics_enabled,
-        .enabled = 1,
+        .enabled = 0,
         .thread = NULL,
         .init_routine = NULL,
         .start_routine = global_statistics_main
@@ -69,10 +69,10 @@ const struct netdata_static_thread static_threads_common[] = {
     {
         .name = "STATS_WORKERS",
         .config_section = CONFIG_SECTION_PLUGINS,
-        .config_name = "netdata monitoring",
+        .config_name = "netdata monitoring extended",
         .env_name = "NETDATA_INTERNALS_MONITORING",
         .global_variable = &global_statistics_enabled,
-        .enabled = 1,
+        .enabled = 0,
         .thread = NULL,
         .init_routine = NULL,
         .start_routine = global_statistics_workers_main
@@ -80,10 +80,10 @@ const struct netdata_static_thread static_threads_common[] = {
     {
         .name = "STATS_SQLITE3",
         .config_section = CONFIG_SECTION_PLUGINS,
-        .config_name = "netdata monitoring",
+        .config_name = "netdata monitoring extended",
         .env_name = "NETDATA_INTERNALS_MONITORING",
         .global_variable = &global_statistics_enabled,
-        .enabled = 1,
+        .enabled = 0,
         .thread = NULL,
         .init_routine = NULL,
         .start_routine = global_statistics_sqlite3_main
@@ -195,6 +195,15 @@ const struct netdata_static_thread static_threads_common[] = {
         .init_routine = NULL,
         .start_routine = profile_main
     },
+    {
+        .name = "DYNCFG",
+        .config_section = NULL,
+        .config_name = NULL,
+        .enabled = 1,
+        .thread = NULL,
+        .init_routine = NULL,
+        .start_routine = dyncfg_main
+    },
 
     // terminator
     {
