@@ -53,12 +53,12 @@ static inline void modify_bit(unsigned *x, unsigned pos, uint8_t val)
         *x |= 1U << pos;
         break;
     default:
-        error("modify_bit() called with invalid argument.");
+        netdata_log_error("modify_bit() called with invalid argument.");
         break;
     }
 }
 
-#define RRDENG_PATH_MAX (4096)
+#define RRDENG_PATH_MAX (FILENAME_MAX + 1)
 
 /* returns old *ptr value */
 static inline unsigned long ulong_compare_and_swap(volatile unsigned long *ptr,
