@@ -9,8 +9,8 @@ static pthread_attr_t *netdata_threads_attr = NULL;
 
 typedef struct {
     void *arg;
-    pthread_t *thread;
     char tag[NETDATA_THREAD_NAME_MAX + 1];
+    SPINLOCK detach_lock;
     void *(*start_routine) (void *);
     NETDATA_THREAD_OPTIONS options;
 } NETDATA_THREAD;
@@ -123,17 +123,17 @@ size_t netdata_threads_init(void) {
     // --------------------------------------------------------------------
     // get the required stack size of the threads of netdata
 
-    netdata_threads_attr = callocz(1, sizeof(pthread_attr_t));
-    i = pthread_attr_init(netdata_threads_attr);
-    if(i != 0)
-        fatal("pthread_attr_init() failed with code %d.", i);
+    if(!netdata_threads_attr) {
+        netdata_threads_attr = callocz(1, sizeof(pthread_attr_t));
+        i = pthread_attr_init(netdata_threads_attr);
+        if (i != 0)
+            fatal("pthread_attr_init() failed with code %d.", i);
+    }
 
     size_t stacksize = 0;
     i = pthread_attr_getstacksize(netdata_threads_attr, &stacksize);
     if(i != 0)
         fatal("pthread_attr_getstacksize() failed with code %d.", i);
-    else
-        debug(D_OPTIONS, "initial pthread stack size is %zu bytes", stacksize);
 
     return stacksize;
 }
@@ -150,12 +150,23 @@ void netdata_threads_init_after_fork(size_t stacksize) {
     if(netdata_threads_attr && stacksize > (size_t)PTHREAD_STACK_MIN) {
         i = pthread_attr_setstacksize(netdata_threads_attr, stacksize);
         if(i != 0)
-            error("pthread_attr_setstacksize() to %zu bytes, failed with code %d.", stacksize, i);
+            nd_log(NDLS_DAEMON, NDLP_WARNING, "pthread_attr_setstacksize() to %zu bytes, failed with code %d.", stacksize, i);
         else
-            info("Set threads stack size to %zu bytes", stacksize);
+            nd_log(NDLS_DAEMON, NDLP_DEBUG, "Set threads stack size to %zu bytes", stacksize);
     }
     else
-        error("Invalid pthread stacksize %zu", stacksize);
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "Invalid pthread stacksize %zu", stacksize);
+}
+
+// ----------------------------------------------------------------------------
+// threads init for external plugins
+
+void netdata_threads_init_for_external_plugins(size_t stacksize) {
+    size_t default_stacksize = netdata_threads_init();
+    if(default_stacksize < 1 * 1024 * 1024)
+        default_stacksize = 1 * 1024 * 1024;
+
+    netdata_threads_init_after_fork(stacksize ? stacksize : default_stacksize);
 }
 
 // ----------------------------------------------------------------------------
@@ -165,16 +176,19 @@ void rrdset_thread_rda_free(void);
 void sender_thread_buffer_free(void);
 void query_target_free(void);
 void service_exits(void);
+void rrd_collector_finished(void);
 
 static void thread_cleanup(void *ptr) {
     if(netdata_thread != ptr) {
         NETDATA_THREAD *info = (NETDATA_THREAD *)ptr;
-        error("THREADS: internal error - thread local variable does not match the one passed to this function. Expected thread '%s', passed thread '%s'", netdata_thread->tag, info->tag);
+        nd_log(NDLS_DAEMON, NDLP_ERR, "THREADS: internal error - thread local variable does not match the one passed to this function. Expected thread '%s', passed thread '%s'", netdata_thread->tag, info->tag);
     }
+    spinlock_lock(&netdata_thread->detach_lock);
 
     if(!(netdata_thread->options & NETDATA_THREAD_OPTION_DONT_LOG_CLEANUP))
-        info("thread with task id %d finished", gettid());
+        nd_log(NDLS_DAEMON, NDLP_DEBUG, "thread with task id %d finished", gettid());
 
+    rrd_collector_finished();
     sender_thread_buffer_free();
     rrdset_thread_rda_free();
     query_target_free();
@@ -184,31 +198,35 @@ static void thread_cleanup(void *ptr) {
 
     netdata_thread->tag[0] = '\0';
 
+    spinlock_unlock(&netdata_thread->detach_lock);
     freez(netdata_thread);
     netdata_thread = NULL;
 }
 
-static void thread_set_name_np(NETDATA_THREAD *nt) {
+void netdata_thread_set_tag(const char *tag) {
+    if(!tag || !*tag)
+        return;
 
-    if (nt && nt->tag[0]) {
-        int ret = 0;
+    int ret = 0;
 
-        char threadname[NETDATA_THREAD_NAME_MAX+1];
-        strncpyz(threadname, nt->tag, NETDATA_THREAD_NAME_MAX);
+    char threadname[NETDATA_THREAD_NAME_MAX+1];
+    strncpyz(threadname, tag, NETDATA_THREAD_NAME_MAX);
 
 #if defined(__FreeBSD__)
-        pthread_set_name_np(pthread_self(), threadname);
+    pthread_set_name_np(pthread_self(), threadname);
 #elif defined(__APPLE__)
-        ret = pthread_setname_np(threadname);
+    ret = pthread_setname_np(threadname);
 #else
-        ret = pthread_setname_np(pthread_self(), threadname);
+    ret = pthread_setname_np(pthread_self(), threadname);
 #endif
 
-        if (ret != 0)
-            error("cannot set pthread name of %d to %s. ErrCode: %d", gettid(), threadname, ret);
-        else
-            info("set name of thread %d to %s", gettid(), threadname);
+    if (ret != 0)
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot set pthread name of %d to %s. ErrCode: %d", gettid(), threadname, ret);
+    else
+        nd_log(NDLS_DAEMON, NDLP_DEBUG, "set name of thread %d to %s", gettid(), threadname);
 
+    if(netdata_thread) {
+        strncpyz(netdata_thread->tag, threadname, sizeof(netdata_thread->tag) - 1);
     }
 }
 
@@ -219,18 +237,18 @@ void uv_thread_set_name_np(uv_thread_t ut, const char* name) {
     strncpyz(threadname, name, NETDATA_THREAD_NAME_MAX);
 
 #if defined(__FreeBSD__)
-    pthread_set_name_np(ut, threadname);
+    pthread_set_name_np(ut ? ut : pthread_self(), threadname);
 #elif defined(__APPLE__)
     // Apple can only set its own name
     UNUSED(ut);
 #else
-    ret = pthread_setname_np(ut, threadname);
+    ret = pthread_setname_np(ut ? ut : pthread_self(), threadname);
 #endif
 
     thread_name_get(true);
 
     if (ret)
-        info("cannot set libuv thread name to %s. Err: %d", threadname, ret);
+        nd_log(NDLS_DAEMON, NDLP_NOTICE, "cannot set libuv thread name to %s. Err: %d", threadname, ret);
 }
 
 void os_thread_get_current_name_np(char threadname[NETDATA_THREAD_NAME_MAX + 1])
@@ -247,15 +265,15 @@ static void *netdata_thread_init(void *ptr) {
     netdata_thread = (NETDATA_THREAD *)ptr;
 
     if(!(netdata_thread->options & NETDATA_THREAD_OPTION_DONT_LOG_STARTUP))
-        info("thread created with task id %d", gettid());
+        nd_log(NDLS_DAEMON, NDLP_DEBUG, "thread created with task id %d", gettid());
 
     if(pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL) != 0)
-        error("cannot set pthread cancel type to DEFERRED.");
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot set pthread cancel type to DEFERRED.");
 
     if(pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL) != 0)
-        error("cannot set pthread cancel state to ENABLE.");
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot set pthread cancel state to ENABLE.");
 
-    thread_set_name_np(ptr);
+    netdata_thread_set_tag(netdata_thread->tag);
 
     void *ret = NULL;
     pthread_cleanup_push(thread_cleanup, ptr);
@@ -266,25 +284,28 @@ static void *netdata_thread_init(void *ptr) {
 }
 
 int netdata_thread_create(netdata_thread_t *thread, const char *tag, NETDATA_THREAD_OPTIONS options, void *(*start_routine) (void *), void *arg) {
-    NETDATA_THREAD *info = mallocz(sizeof(NETDATA_THREAD));
+    NETDATA_THREAD *info = callocz(1, sizeof(NETDATA_THREAD));
     info->arg = arg;
-    info->thread = thread;
     info->start_routine = start_routine;
     info->options = options;
     strncpyz(info->tag, tag, NETDATA_THREAD_NAME_MAX);
 
+    spinlock_init(&info->detach_lock);
+    spinlock_lock(&info->detach_lock);
+
     int ret = pthread_create(thread, netdata_threads_attr, netdata_thread_init, info);
     if(ret != 0)
-        error("failed to create new thread for %s. pthread_create() failed with code %d", tag, ret);
+        nd_log(NDLS_DAEMON, NDLP_ERR, "failed to create new thread for %s. pthread_create() failed with code %d", tag, ret);
 
     else {
         if (!(options & NETDATA_THREAD_OPTION_JOINABLE)) {
             int ret2 = pthread_detach(*thread);
             if (ret2 != 0)
-                error("cannot request detach of newly created %s thread. pthread_detach() failed with code %d", tag, ret2);
+                nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot request detach of newly created %s thread. pthread_detach() failed with code %d", tag, ret2);
         }
     }
 
+    spinlock_unlock(&info->detach_lock);
     return ret;
 }
 
@@ -298,9 +319,9 @@ int netdata_thread_cancel(netdata_thread_t thread) {
     int ret = pthread_cancel(thread);
     if(ret != 0)
 #ifdef NETDATA_INTERNAL_CHECKS
-        error("cannot cancel thread. pthread_cancel() failed with code %d at %d@%s, function %s()", ret, line, file, function);
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot cancel thread. pthread_cancel() failed with code %d at %d@%s, function %s()", ret, line, file, function);
 #else
-        error("cannot cancel thread. pthread_cancel() failed with code %d.", ret);
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot cancel thread. pthread_cancel() failed with code %d.", ret);
 #endif
 
     return ret;
@@ -312,7 +333,7 @@ int netdata_thread_cancel(netdata_thread_t thread) {
 int netdata_thread_join(netdata_thread_t thread, void **retval) {
     int ret = pthread_join(thread, retval);
     if(ret != 0)
-        error("cannot join thread. pthread_join() failed with code %d.", ret);
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot join thread. pthread_join() failed with code %d.", ret);
 
     return ret;
 }
@@ -320,7 +341,7 @@ int netdata_thread_join(netdata_thread_t thread, void **retval) {
 int netdata_thread_detach(pthread_t thread) {
     int ret = pthread_detach(thread);
     if(ret != 0)
-        error("cannot detach thread. pthread_detach() failed with code %d.", ret);
+        nd_log(NDLS_DAEMON, NDLP_WARNING, "cannot detach thread. pthread_detach() failed with code %d.", ret);
 
     return ret;
 }
